@@ -8,6 +8,11 @@ extern "C" {
 #[cfg(feature = "foo")]
 #[no_mangle]
 pub extern "C" fn rust_func() -> i32 {
+    #[cfg(unix)]
+    {
+        extern crate unixdep;
+        assert!(unixdep::only_on_unix() == 0);
+    }
     assert!(common::common_func() == 0);
     assert!(libothername::stuff() == 42);
     let v: i32;
