@@ -96,7 +96,7 @@ class PlatformAgnosticTests(BasePlatformTests):
         self.init(testdir, override_envvars={'PKG_CONFIG': 'notfound'})
 
     def test_vala_target_with_internal_glib(self):
-        testdir = os.path.join(self.unit_test_dir, '130 vala internal glib')
+        testdir = os.path.join(self.unit_test_dir, '131 vala internal glib')
         for run in [{ 'version': '2.84.4', 'expected': '2.84'}, { 'version': '2.85.2', 'expected': '2.84' }]:
             self.new_builddir()
             self.init(testdir, extra_args=[f'-Dglib-version={run["version"]}'])
@@ -296,10 +296,10 @@ class PlatformAgnosticTests(BasePlatformTests):
                 data = json.load(f)['meson']
 
         with open(os.path.join(testdir, 'expected_mods.json'), encoding='utf-8') as f:
-            expected = json.load(f)['meson']['modules']
+            expected = json.load(f)['meson']
 
-        self.assertEqual(data['modules'], expected)
-        self.assertEqual(data['count'], 70)
+        self.assertEqual(data['modules'], expected['modules'])
+        self.assertEqual(data['count'], expected['count'])
 
     def test_meson_package_cache_dir(self):
         # Copy testdir into temporary directory to not pollute meson source tree.
@@ -565,3 +565,18 @@ class PlatformAgnosticTests(BasePlatformTests):
         """Mixing unity and unity_size as long and short options should work."""
         testdir = self.copy_srcdir(os.path.join(self.common_test_dir, '1 trivial'))
         self.init(testdir, extra_args=['-Dunity=on', '--unity-size=123'])
+
+    def test_readonly_sourcedir(self) -> None:
+        """Test building with read-only source directory."""
+        testdir = self.copy_srcdir(os.path.join(self.common_test_dir, '233 wrap case'))
+
+        # Make the source directory and all its contents read-only recursively
+        # Keep execute permission on directories
+        for dir, _, files in os.walk(testdir):
+            os.chmod(dir, 0o555)
+            for file in files:
+                filepath = os.path.join(dir, file)
+                os.chmod(filepath, 0o444)
+
+        self.init(testdir)
+        self.build()
